use axum::{
    extract::rejection::JsonRejection,
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use serde_json::json;
use thiserror::Error;

/// Result type for HTTP requests
pub type Result<T> = core::result::Result<T, AppError>;

/// Error type that can be returned from HTTP requests
#[derive(Error, Debug)]
pub enum AppError {
    /// Client error (e.g. bad input, not found)
    #[error("client error: <{code}> {message}")]
    Client {
        status: StatusCode,
        code: &'static str,
        message: String,
    },

    /// Server error caused by Docker API errors
    #[error("docker error: {0}")]
    DockerError(#[from] bollard::errors::Error),

    /// Server error caused by file operation errors
    #[error("file error: {0}")]
    FileError(#[from] std::io::Error),

    /// Server error caused by unexpected internal errors
    #[error("unexpected internal error: {0}")]
    Internal(#[from] anyhow::Error),

    /// Errors caused by JSON parsing errors (both incoming and outgoing)
    #[error("incoming JSON format error: {0}")]
    JSONFormat(#[from] JsonRejection),
}

pub(super) struct ErrorInfo {
    pub(super) status: StatusCode,
    pub(super) code: String,
    pub(super) message: String,
}

impl IntoResponse for AppError {
    fn into_response(self) -> Response {
        let info = match self {
            AppError::Internal(err) => ErrorInfo {
                status: StatusCode::INTERNAL_SERVER_ERROR,
                code: "server-error".to_string(),
                message: err.to_string(),
            },
            AppError::FileError(err) => ErrorInfo {
                status: StatusCode::INTERNAL_SERVER_ERROR,
                code: "file-error".to_string(),
                message: err.to_string(),
            },
            AppError::Client {
                status,
                code,
                message,
            } => ErrorInfo {
                status,
                code: code.to_string(),
                message: message.to_string(),
            },
            AppError::DockerError(err) => match err {
                bollard::errors::Error::DockerResponseServerError {
                    status_code,
                    message,
                } => ErrorInfo {
                    status: StatusCode::from_u16(status_code).unwrap(),
                    code: "docker-error".to_string(),
                    message,
                },
                _ => ErrorInfo {
                    status: StatusCode::INTERNAL_SERVER_ERROR,
                    code: "docker-error".to_string(),
                    message: err.to_string(),
                },
            },
            AppError::JSONFormat(rejection) => {
                let status = match rejection {
                    JsonRejection::JsonDataError(_) => StatusCode::UNPROCESSABLE_ENTITY,
                    JsonRejection::JsonSyntaxError(_) => StatusCode::BAD_REQUEST,
                    JsonRejection::MissingJsonContentType(_) => StatusCode::UNSUPPORTED_MEDIA_TYPE,
                    _ => StatusCode::INTERNAL_SERVER_ERROR,
                };
                ErrorInfo {
                    status,
                    code: "invalid-body".to_string(),
                    message: rejection.body_text().clone(),
                }
            }
        };

        (
            info.status,
            Json(json!({"code": info.code, "message": info.message})),
        )
            .into_response()
    }
}
