use anyhow::Result;
use bollard::Docker;
use clap::{arg, Parser};
use std::{net::SocketAddr, path::PathBuf};
use tokio::{fs, net::TcpListener};

mod http;
mod node;
mod route;

/// HTTP API for managing stax deployments
#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// Path to checked out repository of the containerlib project
    #[arg(short = 'd', long = "stax-dir", env = "STAX_DIR")]
    stack_dir: PathBuf,

    /// Address:port to bind
    #[arg(short, long, default_value = "0.0.0.0:3000", env = "STAX_BIND")]
    bind: SocketAddr,

    /// Path to docker CLI binary
    #[arg(long = "docker-bin", env = "STAX_DOCKER_BIN", default_value = "docker")]
    docker_binary: PathBuf,

    /// Path to deno binary
    #[arg(long = "deno-bin", env = "STAX_DENO_BIN", default_value = "deno")]
    deno_binary: PathBuf,
}

#[derive(Clone)]
pub struct AppState {
    pub stack_dir: PathBuf,
    pub docker_bin: PathBuf,
    pub deno_bin: PathBuf,
    pub docker: Docker,
}

#[tokio::main]
async fn main() -> Result<()> {
    // Initialize logging and env
    _ = dotenvy::dotenv();
    tracing_subscriber::fmt::init();

    // Parse args
    let args = Args::parse();
    tracing::info!("listening on {}", &args.bind);

    // Try to connect to docker server
    let docker = Docker::connect_with_local_defaults()?;

    // Ping to make sure it works
    let version = docker.version().await?;
    tracing::info!(
        "docker version: {}",
        version.version.expect("failed to get docker version")
    );

    // Make sure stack arg exists and is properly initialized
    fs::create_dir_all(&args.stack_dir)
        .await
        .expect("failed to create stack dir");

    let state = AppState {
        stack_dir: args.stack_dir,
        docker_bin: args.docker_binary,
        deno_bin: args.deno_binary,
        docker,
    };

    let app = route::router().with_state(state);

    let listener = TcpListener::bind(&args.bind)
        .await
        .expect("failed to bind to address");
    axum::serve(listener, app.into_make_service()).await?;

    Ok(())
}
